<?php class dataset extends baseobject {

    protected static $fields=array(
        'name'=>validator::REQUIRED,
        'crystalid'=>array(validator::REQUIRED, validator::INTEGER),
        'diffractionrequestid'=>validator::INTEGER,
        'pdbdepositionid'=>validator::INTEGER,
        'remotedatasetid'=>validator::INTEGER,
        'starrating'=>validator::INTEGER,
        'datalocation'=>validator::REQUIRED,
        'description'=>validator::ANY,
        'beamlineid'=>validator::INTEGER,
        'detectormanufacturer'=>validator::ANY,
        'detectormodel'=>validator::ANY,
        'detectortype'=>validator::ANY,
		'detectormode'=>validator::ANY,
		'bestresolution'=>validator::FLOAT,
        'wavelength'=>validator::FLOAT
    );

    protected static $aliasedColumns=array(
        'proteinacronym',
        'shipmentid'
    );

    protected static $helpTexts=array(
        'crystalid'=>'The crystal from which this dataset came',
        'diffractionrequestid'=>'The diffraction request from which this dataset came',
        'starrating'=>'A subjective score of this dataset\'s importance',
        'remotedatasetid'=>'The ID of this dataset in the synchrotron database',
        'datalocation'=>'The path, URL, or DOI where the raw data can be found'
    );

    protected static $normalSelect='SELECT dataset.*, ROUND(dataset.bestresolution, 2) AS bestresolution,
        ROUND(dataset.wavelength, 4) AS wavelength,
       protein.proteinacronym, protein.name AS proteinname, protein.id AS proteinid, 
       construct.name AS constructname, construct.id AS constructid,
       crystal.name AS crystalname, crystal.dropimageid, pdbdeposition.name AS pdbcode, project.owner AS projectownerid,
            project.name AS projectname, project.isarchived as isarchived,
            diffractionrequest.crystalidatremotefacility, diffractionrequest.crystalurlatremotefacility,
            shipment.id AS shipmentid, shipment.name AS shipmentname, 
            shipment.proposalname as proposalname, shipment.sessionname AS sessionname,
            shipment.idatremotefacility AS shipmentidatremotefacility, shipment.urlatremotefacility AS shipmenturlatremotefacility,
            shipmentdestination.id AS shipmentdestinationid, shipmentdestination.name AS shipmentdestinationname,
            shipmentdestination.shipmenthandler
        FROM dataset JOIN crystal ON dataset.crystalid=crystal.id
            JOIN welldrop ON crystal.welldropid = welldrop.id
            JOIN construct ON welldrop.constructid = construct.id
            JOIN protein ON construct.proteinid = protein.id
            JOIN project on crystal.projectid=project.id 
            LEFT JOIN pdbdeposition ON dataset.pdbdepositionid=pdbdeposition.id 
            LEFT JOIN diffractionrequest ON diffractionrequest.id=dataset.diffractionrequestid
            LEFT JOIN shipment ON shipment.id=diffractionrequest.shipmentid
            LEFT JOIN shipmentdestination on shipment.shipmentdestinationid=shipmentdestination.id
        WHERE 1=1
    ';

    protected static $adminSelect='SELECT dataset.*, ROUND(dataset.bestresolution, 2) AS bestresolution, 
	        ROUND(dataset.wavelength, 4) AS wavelength,
           crystal.name AS crystalname, crystal.dropimageid, pdbdeposition.name AS pdbcode, project.owner AS projectownerid,
         protein.proteinacronym, protein.name AS proteinname, protein.id AS proteinid, 
            project.name AS projectname, project.isarchived as isarchived,
           construct.name AS constructname, construct.id AS constructid,
            diffractionrequest.crystalidatremotefacility, diffractionrequest.crystalurlatremotefacility,
            shipment.id AS shipmentid, shipment.name AS shipmentname, 
            shipment.proposalname as proposalname, shipment.sessionname AS sessionname,
            shipment.idatremotefacility AS shipmentidatremotefacility, shipment.urlatremotefacility AS shipmenturlatremotefacility,
            shipmentdestination.id AS shipmentdestinationid, shipmentdestination.name AS shipmentdestinationname,
            shipmentdestination.shipmenthandler
        FROM dataset JOIN crystal ON dataset.crystalid=crystal.id
            JOIN welldrop ON crystal.welldropid = welldrop.id
            JOIN construct ON welldrop.constructid = construct.id
            JOIN protein ON construct.proteinid = protein.id
            JOIN project on crystal.projectid=project.id 
            LEFT JOIN pdbdeposition ON dataset.pdbdepositionid=pdbdeposition.id 
            LEFT JOIN diffractionrequest ON diffractionrequest.id=dataset.diffractionrequestid
            LEFT JOIN shipment ON shipment.id=diffractionrequest.shipmentid
            LEFT JOIN shipmentdestination on shipment.shipmentdestinationid=shipmentdestination.id
        WHERE 1=1
    ';

    public static function create($request = array()){
		shipment::setShipperAdmin();
        $crystalId=$request['crystalid'];
        if(!$crystalId){ throw new BadRequestException("Crystal ID not specified"); }
        $crystal=crystal::getById($crystalId);
        if(!$crystal){ throw new NotFoundException("Crystal not found"); }
        if(1*session::getUserId()!==1*$crystal['projectownerid'] && session::isShipper() && !session::isAdmin()){
            throw new ForbiddenException('Only the project owner can manage datasets');
        }
        $request['projectid']=$crystal['projectid'];
        $request['name']='dataset'.microtime(true);
        $ret=parent::create($request);
		shipment::revertShipperAdmin();
        return $ret;
    }

    public static function update($id, $request=array()){
		shipment::setShipperAdmin();
        $dataset=static::getById($id);
        if(!$dataset){ throw new NotFoundException('Dataset does not exist or you do not have permission to see it'); }
        if(!session::isShipper() && !session::isAdmin() && (int)($dataset['projectownerid'])!==session::getUserId()){
            throw new ForbiddenException('Only the project owner can modify datasets');
        }
        $ret=parent::update($id, $request);
		shipment::revertShipperAdmin();
        return $ret;
    }

    public static function delete($id){
        $deposition=static::getById($id);
        if(!$deposition || session::isAdmin()){
            return parent::delete($id);
        }
        $crystal=crystal::getById($deposition['crystalid']);
        if(!$crystal || (session::getUserId()!==1*$crystal['projectownerid'] && !session::isAdmin())){
            throw new ForbiddenException('Only the project owner can manage datasets');
        }
        return parent::delete($id);
    }

    /**
     * @return bool
     * @throws NotFoundException
     */
    public static function canCreate(){
        return true; //but checked in create()
    }

    /**
     * @param $id
     * @return bool
     * @throws NotFoundException
     */
    public static function canUpdate($id){
        return true; //but checked in create()
    }

    /**
     * @param $id
     * @param array $request
     * @return array
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function getautoprocessingresults($id, $request=array()){
        return autoprocessingresult::getByProperty('datasetid', $id, $request);
    }

    /**
     * @param $id
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function updateFileNoteAndRecordingProjectIds($id){
        parent::updateFileNoteAndRecordingProjectIds($id);
    }

    /**
	 * Sets the "best resolution" field for this dataset, based on its autoprocessing results.
	 * Rationale: 2- and 1-star rated results are worse than unrated, so:
	 * - if there are any 5-star results, set the smallest resolution number
	 * - if not, look for 4-star, then 3-star, then unrated, then 2-star, then 1-star
     * @throws ServerException
     * @throws NotFoundException
     * @throws BadRequestException
     * @throws ForbiddenException
     */
    public static function setBestResolutionFromAutoProcessingResults($id){
        $results=static::getautoprocessingresults($id);
        if(!$results){ return; }
        $bestResolution=999999;
		foreach ([5,4,3,0,2,1] as $starRating) { //because 1 and 2 star are worse than unrated (0)
			$autoprocessingResults=autoprocessingresult::getByProperties([
				'datasetid'=>$id,
				'starrating'=>$starRating
			]);
			if($autoprocessingResults){
				foreach ($autoprocessingResults['rows'] as $autoprocessingResult){
					$resolution = 1 * $autoprocessingResult['bestresolution'];
					if ($resolution !== 0 && $resolution < $bestResolution) {
						$bestResolution = $resolution;
					}
				}
			}
			if(999999!==$bestResolution){ break; }
		}
        if(999999!=$bestResolution){
            dataset::update($id, ['bestresolution'=>$bestResolution]);
        }
    }

}

